<template>
  <AdminLayout>
    <DashboardHeader>
      <DashboardTitle>{{ $t("app.admin.users.title") }}</DashboardTitle>
    </DashboardHeader>
    <DashboardBody>
      <ListActions>
        <FiltersSearch v-model="filters.search" />
        <ButtonGroup>
          <FiltersSelect
            v-model="filters"
            @clear="filters.clear"
            @updated="fetchUsers"
          >
            <FormInput id="filter_name">
              <FormLabel>{{ $t("app.admin.users.filters.name") }}</FormLabel>
              <FormText
                v-model="filters.name"
                name="name"
              />
            </FormInput>
            <FormInput id="filter_email">
              <FormLabel>{{ $t("app.admin.users.filters.email") }}</FormLabel>
              <FormText
                v-model="filters.email"
                name="email"
              />
            </FormInput>
          </FiltersSelect>
          <RouterLink
            v-if="hasPermission(Permission.ADMIN_USERS_CREATE)"
            :to="{ name: 'admin.users.create' }"
            class="btn btn-primary"
          >
            <ButtonIcon :icon="PlusRegularIcon">
              {{ $t("app.admin.users.create.title") }}
            </ButtonIcon>
          </RouterLink>
        </ButtonGroup>
      </ListActions>
      <template v-if="usersFetch.hasErrors">
        <ErrorState />
      </template>
      <template v-else>
        <ListGroup :is-loading="usersFetch.isFetching">
          <ListTable>
            <ListTableHeader>
              <ListTableRow>
                <ListTableCell primary>
                  {{ $t("app.admin.users.form.user") }}
                </ListTableCell>
                <ListTableCell>{{ $t("app.admin.users.form.user_role_id") }}</ListTableCell>
                <ListTableCell>{{ $t("app.admin.users.form.created_at") }}</ListTableCell>
              </ListTableRow>
            </ListTableHeader>
            <ListTableBody>
              <component
                :is="hasPermission(Permission.ADMIN_USERS_UPDATE) ? ListTableRouterLinkRow : ListTableRow"
                v-for="user in collection.data"
                :key="user.id"
                :to="hasPermission(Permission.ADMIN_USERS_UPDATE) ? { name: 'admin.users.update', params: { id: user.id } } : undefined"
              >
                <ListTableCell>
                  <div class="flex items-center gap-2.5">
                    <UserAvatar
                      :user-name="user.name"
                      class="inline-block h-9 w-9 rounded-full"
                    />
                    <div class="flex flex-col text-sm text-gray-800">
                      <div>{{ user.name }}</div>
                      <div class="text-sm">
                        {{ user.email }}
                      </div>
                    </div>
                  </div>
                </ListTableCell>
                <ListTableCell>
                  <UserRoleTooltip
                    v-if="user.user_role"
                    :user-role="user.user_role"
                  />
                  <template v-else>
                    {{ $t("app.admin.user_roles.without") }}
                  </template>
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(user.created_at).toLocaleString() }}
                </ListTableCell>
              </component>
            </ListTableBody>
          </ListTable>
          <ListEmpty
            v-if="collection.empty() && !usersFetch.isFirthFetch"
            :icon="TeamIllustration"
            description="app.admin.users.empty.description"
            title="app.admin.users.empty.title"
          />
        </ListGroup>
        <Pagination
          :collection="collection"
          @change-page="filters.changePage"
        />
      </template>
    </DashboardBody>
  </AdminLayout>
  <RouterView @fetch="fetchUsers()" />
</template>

<script lang="ts" setup>
import AdminLayout from "@/js/contexts/admin/shared/ui/layouts/AdminLayout.vue";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import useCollection from "@/js/contexts/shared/composables/useCollection";
import type { User } from "@/js/contexts/shared/models/User";
import type PaginatedCollection from "@/js/contexts/shared/models/PaginatedCollection";
import { pickBy } from "lodash-es";
import FormText from "@/js/contexts/form/ui/components/FormText.vue";
import useFilters from "@/js/contexts/shared/composables/useFilters";
import DashboardHeader from "@/js/contexts/shared/ui/components/dashboard/DashboardHeader.vue";
import DashboardTitle from "@/js/contexts/shared/ui/components/dashboard/DashboardTitle.vue";
import DashboardBody from "@/js/contexts/shared/ui/components/dashboard/DashboardBody.vue";
import FiltersSearch from "@/js/contexts/shared/ui/components/filters/FiltersSearch.vue";
import ListActions from "@/js/contexts/shared/ui/components/list/ListActions.vue";
import FiltersSelect from "@/js/contexts/shared/ui/components/filters/FiltersSelect.vue";
import FormInput from "@/js/contexts/form/ui/components/FormInput.vue";
import FormLabel from "@/js/contexts/form/ui/components/FormLabel.vue";
import PlusRegularIcon from "@/svg/icons/plus-regular.svg?component";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import ListGroup from "@/js/contexts/shared/ui/components/list/ListGroup.vue";
import ListEmpty from "@/js/contexts/shared/ui/components/list/ListEmpty.vue";
import TeamIllustration from "@/svg/illustrations/team.svg?component";
import UserAvatar from "@/js/contexts/shared/ui/components/avatar/UserAvatar.vue";
import UserRoleTooltip from "@/js/contexts/admin/user-roles/ui/components/UserRoleTooltip.vue";
import ListTable from "@/js/contexts/shared/ui/components/list/ListTable.vue";
import ListTableHeader from "@/js/contexts/shared/ui/components/list/ListTableHeader.vue";
import ListTableCell from "@/js/contexts/shared/ui/components/list/ListTableCell.vue";
import ListTableBody from "@/js/contexts/shared/ui/components/list/ListTableBody.vue";
import ListTableRow from "@/js/contexts/shared/ui/components/list/ListTableRow.vue";
import ListTableRouterLinkRow from "@/js/contexts/shared/ui/components/list/ListTableRouterLinkRow.vue";
import Pagination from "@/js/contexts/shared/ui/components/pagination/Pagination.vue";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import { Permission } from "@/js/contexts/shared/models/Permission";
import ButtonGroup from "@/js/contexts/shared/ui/components/button/ButtonGroup.vue";

const { t } = useI18n();
const { hasPermission } = useAuth();

useHead({
  title: t("app.admin.users.title"),
});

const usersFetch = useFetch("/api/admin/users");
const collection = useCollection<User>();
const filters = useFilters(
  {
    search: "",
    name: "",
    email: "",
  },
  ["search"],
);

onMounted(() => {
  fetchUsers();
});

const fetchUsers = () => {
  usersFetch.get<PaginatedCollection<User>>(pickBy(filters.value.data())).then((response) => {
    collection.fromResponse(response.data);
  });
};
</script>
